<?php

/**
 *
 * @wordpress-plugin
 * Plugin Name:       UBC Event Espresso My Events
 * Plugin URI:        http://ctlt.ubc.ca/
 * Description:       This plugin provides a my-events listings for people to see the events for which they have registered
 * Version:           1.0.2
 * Author:            Richard Tape
 * Author URI:        http://blogs.ubc.ca/mbcx9rvt
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       ubc-event-espresso-my-events
*/

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

class UBC_EE_My_Events {

	/**
	 * $passed_template_args is an array of arguments passed to the internal EE template
	 * loader. We hook in as the template is called and then set it as a class property
	 * so we're able to use it later
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var $passed_template_args
	 */

	static $passed_template_args = false;


	/**
	 * The User ID of the person signed in. Used on the event-registration form
	 * as for some reason this clears the User ID
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (array) $user_id
	 */

	static $user_id = false;


	/**
	 * The user meta key where registrations are kept
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (string) $user_meta_key
	 */

	static $user_meta_key = 'ubc_event_registrations';

	/**
	 * The slug for the my events page
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (string) $my_events_slug
	 */

	static $my_events_slug = '/my-events/';


	/**
	 * Are the manual cancellations active on the front-end?
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @var (bool) $cancellations_active
	 */
	static $cancellations_active = false;

	/**
	 * Set up our actions and filters
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */



	public function init() {

		// Allow us to have access to the template args used within this class
		add_filter( 'FHEE__EEH_Template__display_template__template_args', array( $this, 'FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args' ) );

		// When someone registers for an event, add it to their user meta, this sets up the user ID for us to use
		add_action( 'wp_ajax_process_reg_step', array( $this, 'wp_ajax_process_reg_step__add_user_meta_on_registration' ) );
		add_action( 'wp_ajax_nopriv_process_reg_step', array( $this, 'wp_ajax_process_reg_step__add_user_meta_on_registration' ) );

		// Upon the processing of the registration, let's add the user meta
		add_filter( 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process', array( $this, 'FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_user_meta_on_registration' ), 10, 6 );

		// We fire an action in the filter above, let's hook in and add the user meta
		add_action( 'ubc_ee_my_events_pre_reg_process_data', array( $this, 'ubc_ee_my_events_pre_reg_process_data__record_reg_to_user' ), 10, 5 );

		// Output the events in user profile
		add_action( 'show_user_profile', array( $this, 'show_edit_user_profile__add_event_registrations' ) );
		add_action( 'edit_user_profile', array( $this, 'show_edit_user_profile__add_event_registrations' ) );

		// Add a shortcode for the Output
		add_shortcode( 'my_events', array( $this, 'add_shortcode__my_events__display_markup' ) );

		// We need to ensure the User ID is passed with the form
		add_action( 'AHEE__before_spco_whats_next_buttons', array( $this, 'AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field_if_not_present' ), 15, 3 );
		add_action( 'wp_ajax_process_reg_step', array( $this, 'wp_ajax_process_reg_step__access_user_id_from_reg' ) );

		// If a user has already registered for an event, we prevent them from doing so again
		add_filter( 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path', array( $this, 'FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__you_have_already_registered_template' ), 10, 2 );

		// Register our stylesheet. It is enqueued when we need it
		add_action( 'wp_enqueue_scripts', array( $this, 'wp_enqueue_scripts__register_stylesheet' ) );

		// If someone wants to cancel their event registration
		add_action( 'init', array( $this, 'init__cancel_registration_when_user_requests' ) );

		// Also handle when an admin cancels a reg from the back-end
		add_action( 'AHEE__EE_Registration__set_status__after_update', array( $this, 'AHEE__EE_Registration__set_status__after_update__cancel_reg_from_dashboard' ) );

	}/* init() */

	/**
	 * When EE loads a template it can pass arguments to it. We need a nice way to get access to the
	 * tickets for this event when we're adjusting the table headers, so this class method hooks into the
	 * FHEE__EEH_Template__display_template__template_args filter provided by EE core and sets the passed
	 * args as a class argument here, so we can access them.
	 *
	 * @since 1.0.0
	 *
	 * @param array $template_args - The passed template args
	 * @return array $template_args - unmodified template args
	 */

	public function FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args( $template_args ) {

		// We just want to set this to a class variable so we're able to access these args elsehwere
		static::$passed_template_args = $template_args;

		// Return unmodified
		return $template_args;

	}/* FHEE__EEH_Template__display_template__template_args__set_class_properties_for_args() */


	/**
	 * We need access to the User ID throughout this class, so let's ensure we have it
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function wp_ajax_process_reg_step__add_user_meta_on_registration() {

		if ( ! isset( $_POST ) || empty( $_POST ) || ! isset( $_POST['ubc_ctlt_user_id'] ) ) {
			return;
		}

		$user_id = absint( $_POST['ubc_ctlt_user_id'] );

		$user_id = absint( $user_id );

		// Double check for sanity-ization
		if ( ! $user_id ) {
			return;
		}

		static::$user_id = $user_id;

	}/* wp_ajax_process_reg_step__add_user_meta_on_registration() */


	/**
	 * When someone submits a registation for an event, hook in and fire an action if we have a user ID
	 * so we can then associate this event registration with the user signed in
	 *
	 * @since 1.0.0
	 *
	 * @param (bool) $process_reg - Whether to process the registration or not
	 * @param (int) $att_number - The attendance number
	 * @param (object) $registration - The full registration object
	 * @param (array) $registrations - All of the registrations from this transaction
	 * @param (array) $valid_data - The validated data passed to _process_registrations()
	 * @param (object) $attendee_object - The full EE_SPCO_Reg_Step_Attendee_Information object
	 * @return (bool) Whether to carry on processing the registration or not
	 */

	public function FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_user_meta_on_registration( $process_reg, $att_nmbr, $registration, $registrations, $valid_data, $attendee_object ) {
		// Fetch the user ID
		$user_id = static::$user_id;

		$user_id = absint( $user_id );

		// Double check for sanity-ization
		if ( ! $user_id ) {
			return;
		}

		// Should be part of the reg object and will allow us to form a url such as
		// to see confirmation /thank-you/?e_reg_url_link=1-5a574b502a96bbfa2e226a7424687997
		// to resend emails /thank-you/?token=1-5a574b502a96bbfa2e226a7424687997&resend_reg_confirmation=true
		$reg_url_link = $registration->reg_url_link();

		// THe Event ID so we can link back to it and get details
		$event_id = $registration->event_ID();

		// Just double check we have these pieces of info
		if ( ! $reg_url_link || ! $event_id ) {
			return;
		}

		EE_Registry::instance()->load_helper( 'Event_View' );
		$event_object = EEH_Event_View::get_event( $event_id );

		$date_object = $event_object->primary_datetime();
		$event_start_date = $date_object->start_date();

		$venue_objects_array = $event_object->venues();
		$venues = array();

		if ( is_array( $venue_objects_array ) ) {

			foreach ( $venue_objects_array as $venue_id => $venue_object ) {
				$venues[] = $venue_object->name();
			}
		}

		// OK we have the data we need, let's fire an action so we can hook in elsewhere should we need
		do_action( 'ubc_ee_my_events_pre_reg_process_data', esc_html( $reg_url_link ), absint( $event_id ), $registration, $event_start_date, $venues );

		return $process_reg;

	}/* FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_user_meta_on_registration() */


	/**
	 * Add the registration to this user
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $reg_url_link - The somewhat random string generated by EE which allows to us form URLs to link back to the registration
	 * @param (int) $event_id - The ID of the Event that the person has just registered for
	 * @param (object) $registration - The whole event registration object
	 * @param (string) $start_date - Event start date
	 * @param (array) $venues - an array of venues for this event
	 * @return null
	 */

	public function ubc_ee_my_events_pre_reg_process_data__record_reg_to_user( $reg_url_link, $event_id, $registration, $start_date, $venues ) {

		$user_id = static::$user_id;

		$event_id = absint( $event_id );

		static::add_registration_to_user_for_event( $registration, $user_id, $event_id, $venues, $start_date, $reg_url_link );

	}/* ubc_ee_my_events_pre_reg_process_data__record_reg_to_user() */


	/**
	 * Helper method to add a new registration to a user's meta
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $registration  - An array of registration details
	 * @param (int) $user_id - the user to which we're adding this $registration
	 * @param (int) $event_id - The ID of the event for which $user_id has just registered
	 * @param (array) $venues - an array of venues for this event
	 * @param (string) $start_date - the date the event starts
	 * @param (string) $reg_url_link - the unique reg url link for this event
	 * @return (bool) If this action was successful or not
	 */

	public static function add_registration_to_user_for_event( $registration, $user_id = false, $event_id = false, $venues = null, $start_date = null, $reg_url_link = null ) {

		if ( ! $event_id ) {
			return false;
		}

		if ( ! $user_id ) {
			$user_id = get_current_user_id();
		}

		$user_id = absint( $user_id );
		$event_id = absint( $event_id );

		// Fetch the current registrations
		$current_registrations = get_user_meta( $user_id, static::$user_meta_key, true );

		// $current_registrations is an empty string if it's not set
		if ( empty( $current_registrations ) ) {
			$current_registrations = array();
		}

		// We need a reg and session ID. The latter is used as a check when removing
		$registration_id = $registration->ID();
		$session_id = $registration->session_ID();

		// If there's already an entry in this array, we don't add it again
		if ( array_key_exists( $session_id, $current_registrations ) ) {
			return;
		}

		// Build our addition
		$new_reg = array(
			'event_id' 			=> $event_id,
			'reg_url_link' 		=> sanitize_text_field( $reg_url_link ),
			'registered_on' 	=> date( 'Y-m-d H:i:s' ),
			'start_date' 		=> $start_date,
			'venues'			=> $venues,
			'registration_id'	=> $registration_id,
			'session_id'		=> $session_id,
		);

		// And add it
		$current_registrations[ $registration_id ] = $new_reg;

		return update_user_meta( $user_id, static::$user_meta_key, $current_registrations );

	}/* add_registration_to_user() */


	/**
	 * Add the events a user has registered for to the main WP profile for the user
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $user - The current user object
	 * @return null
	 */

	public function show_edit_user_profile__add_event_registrations( $user ) {

		$registered_events = get_user_meta( $user->ID, static::$user_meta_key, true );

		static::my_events_markup( $registered_events );

	}/* show_edit_user_profile__add_event_registrations() */


	/**
	 * Method to generate the generic markup for the My Events display
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $registered_events - The user meta for the events
	 * @param (bool) $with_title - display the title or not
	 * @return string The markup to show
	 */

	private function my_events_markup( $registered_events = false, $with_title = true ) {

		if ( ! $registered_events && is_user_logged_in() ) {
			$registered_events = get_user_meta( get_current_user_id(), static::$user_meta_key, true );
		}

		if ( ! $registered_events ) {
			$registered_events = array();
		}

		?>

		<?php if ( $with_title ) : ?>
		<h3><?php _e( 'Event Registrations', 'ubc-event-espresso-my-events' ); ?></h3>
		<?php endif; ?>
		<p><?php _e( 'These are the events for which you have registered.', 'ubc-event-espresso-my-events' ); ?></p>

		<?php

		if ( empty( $registered_events ) ) {
			_e( 'You have not registered for any events.', 'ubc-event-espresso-my-events' );
			return;
		}

		?>
		<table class="form-table ubc-ee-my-events">
			<thead>
				<tr>
					<th>Event Title</th>
					<th>Venue</th>
					<th>Start Date</th>
					<th>Actions</th>
				</tr>
			</thead>

			<tbody>
		<?php

		EE_Registry::instance()->load_helper( 'Event_View' );

		// The thank you page ID
		$thank_you_page_id = EE_Registry::instance()->CFG->core->thank_you_page_id;
		$thank_you_page_permalink = get_permalink( $thank_you_page_id );
		foreach ( $registered_events as $id => $reg_details ) {

			if ( ! isset( $reg_details['event_id'] ) ) {
				continue;
			}

			$event_id = $reg_details['event_id'];

			// Fetch an Event Object
			$event_object = EEH_Event_View::get_event( $event_id );

			// Fetch details we need about this event
			$event_link = $event_object->get_permalink();
			$event_name = $event_object->name();

			// EEH_Event_View::event_date_as_calendar_page( $event_id );
			?>
			<tr>
				<td class="event-details event-name"><label id="" for=""><?php echo esc_html( $event_name ); ?></label></td>
				<td class="event-details event-venue">
					<p><?php echo sanitize_text_field( implode( ', ', $reg_details['venues'] ) ); ?></p>
				</td>
				<td class="event-details event-start">
					<p><?php echo wp_kses_post( $reg_details['start_date'] ); ?></p>
				</td>
				<td class="event-actions">
					<a href="<?php echo esc_url( $event_link ); ?>" title=""><?php esc_html_e( 'View Event', 'ubc-event-espresso-my-events' ); ?></a> <br />
					<a href="<?php echo esc_url( $thank_you_page_permalink . '?e_reg_url_link=' . $reg_details['reg_url_link'] ); ?>" title=""><?php esc_html_e( 'Edit Event Registration Details', 'ubc-event-espresso-my-events' ); ?></a><br />
					<?php if ( static::$cancellations_active ) : ?>
					<a href="<?php echo esc_url( add_query_arg( array( 'ubc_ee_action' => 'cancel_reg', 'event_id' => $event_id, 'reg_id' => $reg_details['registration_id']  ), static::$my_events_slug ) ); ?>" title=""><?php esc_html_e( 'Cancel Registration', 'ubc-event-espresso-my-events' ); ?></a>
					<?php endif; ?>
				</td>
			</tr>
			<?php

		}

		?>
			</tbody>
		</table>
		<?php

	}/* my_events_markup() */


	/**
	 * Make the my events markup available externally
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $registered_events - The user meta for the events
	 * @param (bool) $with_title - whether to display the title or not
	 * @return string The markup to show
	 */

	public static function output_my_events_markup( $registered_events = false, $with_title = true ) {

		static::my_events_markup( $registered_events, $with_title );

	}/* output_my_events_markup() */


	/**
	 * The shortcode for [my_events]
	 *
	 * @since 1.0.0
	 *
	 * @param (array) $atts - Any shortcode atts passed
	 * @return (string) markup
	 */

	public function add_shortcode__my_events__display_markup( $atts ) {

		if ( ! is_user_logged_in() ) {
			return '<p>You must be signed in to view the events for which you have registered.</p>';
		}

		$user_id = get_current_user_id();

		// Fetch the current registrations
		$current_registrations = get_user_meta( $user_id, static::$user_meta_key, true );

		ob_start();

		static::my_events_markup( $current_registrations );

		$output = ob_get_contents();

		ob_end_clean();

		return $output;

	}/* add_shortcode__my_events__display_markup() */


	/**
	 * We need to add the user ID to the POST submitted when a user registers because for some reason
	 * it's being cleared. We later access this in the AJAX event wp_ajax_process_reg_step__access_user_id_from_reg Callback
	 * which then sets it to a class variable which we're then able to access in
	 * FHEE__EE_SPCO_Reg_Step_Attendee_Information___process_registrations__pre_registration_process__add_extra_user_meta_on_registration
	 * to be able to add some user meta
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $slug - The path to the current step in the form
	 * @param (string) $next_step_slug - The path to the next step in the form
	 * @param (object) $checkout - The entire checkout object
	 * @return null;
	 */

	public function AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field_if_not_present( $slug, $next_step_slug, $checkout ) {

		if ( ! is_user_logged_in() ) {
			return;
		}

		$output = apply_filters( 'ubc_ee_add_output_to_reg_form', '' );

		if ( ! empty( $output ) ) {
			return;
		}

		// Add to the filter to output the user ID in the form
		add_filter( 'ubc_ee_add_output_to_reg_form', array( $this, 'ubc_ee_add_output_to_reg_form__add_user_id_to_form' ) );

	}/* AHEE__before_spco_whats_next_buttons__add_user_id_hidden_field_if_not_present() */


	/**
	 * Add the user ID as a hidden form field to the form
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $output - Any output already added
	 * @return (string) $output modified with the hidden field
	 */

	public function ubc_ee_add_output_to_reg_form__add_user_id_to_form( $output = '' ) {

		$user_id = get_current_user_id();

		$output .= '<input type="hidden" readonly value="' . absint( $user_id ) . '" name="ubc_ctlt_user_id" id="ubc_ctlt_user_id" />';

		return $output;

	}/* ubc_ee_add_output_to_reg_form__add_user_id_to_form() */


	/**
	 * HOLY HACK BATMAN
	 * For some ridiculous reason the user object is getting removed when we submit. So,
	 * we have to hook into the AJAX action and store the user ID that is submitted as a new
	 * hidden field to a class variable. We can then access it when we're setting the user meta
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function wp_ajax_process_reg_step__access_user_id_from_reg() {

		if ( ! isset( $_POST ) || empty( $_POST ) || ! isset( $_POST['ubc_ctlt_user_id'] ) ) {
			return;
		}

		$user_id = absint( $_POST['ubc_ctlt_user_id'] );

		static::$user_id = $user_id;

	}/* wp_ajax_process_reg_step__access_user_id_from_reg() */


	/**
	 * If a user has already registered for an event, we don't show them the ticket selector again
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $template_path - The path to the template. This overwrites the ticket_selector_chart.template.php
	 * @param (object) $event - The event object of the event we're currently on
	 * @return (string) a path to our custom you must log in template
	 */
	public function FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__you_have_already_registered_template( $template_path, $event ) {

		// Not signed in? We don't know which events you have
		if ( ! is_user_logged_in() ) {
			return $template_path;
		}

		// If this is in the admin, we don't want this to happen
		if ( is_admin() ) {
			return $template_path;
		}

		// Fetch user meta
		$registered_events = get_user_meta( get_current_user_id(), static::$user_meta_key, true );

		// Empty? No events registered for then, so Back to normal template
		if ( ! $registered_events || empty( $registered_events ) ) {
			return $template_path;
		}

		if ( ! is_a( $event, 'EE_Event' ) ) {
			return $template_path;
		}

		// Fetch the event ID
		$event_id = $event->ID();

		// Now look in the registered events and see if this ID matches. Default to false
		$has_registered = false;

		foreach ( $registered_events as $key => $event_info ) {

			if ( $event_info['event_id'] == $event_id ) {
				$has_registered = true;
				break;
			}
		}

		if ( $has_registered ) {
			return plugin_dir_path( __FILE__ ) . 'templates/you-have-already-registered.php';
		}

		return $template_path;

	}/* FHEE__EE_Ticket_Selector__display_ticket_selector__template_path__you_have_already_registered_template() */


	public function wp_enqueue_scripts__register_stylesheet() {

		wp_register_style( 'ubc-event-espresso-my-events', plugins_url( 'assets/css/ubc-event-espresso-my-events.css', __FILE__ ) );
		wp_enqueue_style( 'ubc-event-espresso-my-events' );

	}/* wp_enqueue_scripts__register_stylesheet() */


	/**
	 * When someone cancels a registration they're sent to a url such as
	 * /my-events/?ubc_ee_action=cancel_reg&event_id=1165&reg_id=3924
	 * We intercept this, then check that the person trying to do so, can do so
	 * then cancel their registration and remove it from their user meta
	 *
	 * @since 1.0.0
	 *
	 * @param null
	 * @return null
	 */

	public function init__cancel_registration_when_user_requests() {

		if ( ! is_user_logged_in() ) {
			return;
		}

		if ( ! isset( $_GET['ubc_ee_action'] ) || 'cancel_reg' != sanitize_text_field( $_GET['ubc_ee_action'] ) ) {
			return;
		}

		$event_id = absint( $_GET['event_id'] );
		$registration_id = sanitize_text_field( $_GET['reg_id'] );

		$registration = EEM_Registration::instance()->get_one_by_ID( $registration_id );

		// Check that this user is able to cancel this registration.
		$user_id = get_current_user_id();
		$registered_events = get_user_meta( get_current_user_id(), static::$user_meta_key, true );

		if ( ! is_array( $registered_events ) || empty( $registered_events ) ) {
			return;
		}

		// Look through this user's meta for the session ID
		// $session_id_being_deleted = $registration->session_ID();

		if ( ! array_key_exists( $registration_id, $registered_events ) ) {
			wp_redirect( esc_url( add_query_arg( array( 'not_allowed' => 'true' ), static::$my_events_slug ) ) );
			exit;
		}

		// OK, so this user has this event in their list. Let's change the status of the reg and them remove it from their meta
		// Looks like there's something wrong with the EE internals here as the methods are reporting back that it's changed to cancelled, but it hasn't
		// $registration->set_status( 'RCN' );
		//
		// $ticket = $registration->ticket();
		// $ticket->decrease_sold();
		// $ticket->save();
		// $datetimes = $ticket->datetimes();
		// if ( is_array( $datetimes ) ) {
		// 	foreach ( $datetimes as $datetime ) {
		// 		if ( $datetime instanceof EE_Datetime ) {
		// 			$datetime->decrease_sold();
		// 			$datetime->save();
		// 		}
		// 	}
		// }
		//
		// In the interim, we'll have to do things manually. This involves
		// [ ] - esp_registration change STS_ID to `RCN`
		// [ ] - get REG_count from esp_registration so we know how many tickets to manke available
		// [ ] - get the ticket ID and then manually lower the TKT_sold column in esp_ticket

		$removed = static::remove_registration_from_user( $registration_id, $user_id );

		if ( ! $removed ) {
			wp_redirect( esc_url( add_query_arg( array( 'reg_removed' => 'false' ), static::$my_events_slug ) ) );
			exit;
		}

		wp_redirect( esc_url( add_query_arg( array( 'reg_removed' => 'true' ), static::$my_events_slug ) ) );
		exit;

	}/* init__cancel_registration_when_user_requests() */


	/**
	 * Helper method to remove a registration from a user's meta
	 *
	 * @since 1.0.0
	 *
	 * @param (string) $session_id - The session ID to remove
	 * @param (int) $user_id - The user ID from which we are removing this registration
	 * @return (bool)
	 */

	public static function remove_registration_from_user( $session_id, $user_id ) {

		$user_id = absint( $user_id );
		$session_id = sanitize_text_field( $session_id );

		// Only the current user can do this for themselves
		if ( get_current_user_id() !== $user_id ) {
			return false;
		}

		$registered_events = get_user_meta( $user_id, static::$user_meta_key, true );

		// No registrations? No remove-estrations.
		if ( ! $registered_events || ! is_array( $registered_events ) || empty( $registered_events ) ) {
			return false;
		}

		// If the session ID doesn't exist, no dice
		if ( ! array_key_exists( $session_id, $registered_events ) ) {
			return false;
		}

		unset( $registered_events[ $session_id ] );

		update_user_meta( $user_id, static::$user_meta_key, $registered_events );

		return true;

	}/* remove_registration_from_user() */


	/**
	 * When an admin cancels a registration from the dashboard, we need to remove it from user meta
	 *
	 * @since 1.0.0
	 *
	 * @param (object) $registration_object -
	 * @return null
	 */

	public function AHEE__EE_Registration__set_status__after_update__cancel_reg_from_dashboard( $registration_object ) {

		if ( ! is_object( $registration_object ) ) {
			return;
		}

		// Check the status ID is set to cancelled
		$status_id = $registration_object->status_ID();
		if ( ! $status_id || 'RCN' != $status_id ) {
			return;
		}

		// First need to get the ATT_ID from the registration object so we can look in the attendee_meta table
		$attendee_id = $registration_object->attendee_ID();
		$attendee_object = EEM_Attendee::instance()->get_one_by_ID( $attendee_id );

		if ( ! is_object( $attendee_object ) ) {
			return;
		}

		$attendee_email = $attendee_object->email();

		// Now we have the email address of the user who made this reg (unless they specified a different address, in which case there's not a lot we can do
		// aside from looking through every user's meta)
		$user_object = get_user_by( 'email', $attendee_email );

		if ( ! $user_object ) {
			// TODO: Decide what to do here, the email in the registration doesn't match an email in the system
			return;
		}

		$user_id = $user_object->ID;
		$session_id = $registration_object->session_ID();
		$registration_id = $registration_object->ID();

		$result = static::remove_registration_from_user( $registration_id, $user_id );

	}/* AHEE__EE_Registration__set_status__after_update__cancel_reg_from_dashboard() */


}/* class UBC_EE_My_Events */

// Initialize ourselves
add_action( 'plugins_loaded', 'ubc_ee_my_events_initialize' );

/**
 * On plugins_loaded initialize our class to add the ability to have a my-events listing
 *
 * @since 1.0.0
 *
 * @param null
 * @return null
 */

function ubc_ee_my_events_initialize() {

	$UBC_EE_My_Events = new UBC_EE_My_Events;
	$UBC_EE_My_Events->init();

}/* ubc_ctlt_ee_my_events_initialize() */
